package test.jdbc;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

import model.Grade;
import model.Student;
import model.Zodiac;
/**
 * Used Oracle Database, not the Derby DB
 * Look SQL script at src/persistence/jdbc/persisSQL_Script_prepare_tables.sql
 *
 * @version 2011-02-12
 * @author Tsvetan Vasilev
 *
 */
public class TestJDBC {
	private static final String DB_CONNECTION_STRING = "jdbc:oracle:thin:@localhost:1521/xe";
	private static final String DB_USERNAME = "HR";
	private static final String DB_PASSWORD = "hr";
	private static final String SELECT_STUDENTS = "SELECT"
			+ "  S.STUDENT_ID \"ID\", " + " S.NAME \"NAME\", "
			+ " Z.NAME \"ZODIAC\", " + " G.NAME \"GRADE\" " + "FROM STUDENT S "
			+ "INNER JOIN GRADE G ON S.GRADE_ID=G.GRADE_ID "
			+ "INNER JOIN ZODIAC Z ON S.ZODIAC_ID=Z.ZODIAC_ID";

	public static void main(String[] args) {
		// Load the Oracle JDBC driver
		try {
			Class.forName("oracle.jdbc.driver.OracleDriver");
		} catch (ClassNotFoundException ex) {
			ex.printStackTrace();
		}

		Connection connection = null;
		Statement statement = null;
		ResultSet studentsResult = null;
		List<Student> studentsList = null;
		try {
			connection = DriverManager.getConnection(DB_CONNECTION_STRING,
					DB_USERNAME, DB_PASSWORD);

			statement = connection.createStatement();
			studentsResult = statement.executeQuery(SELECT_STUDENTS);
			studentsList = prepareStudentList(studentsResult);

			// print result
			System.out.println(Arrays.deepToString(studentsList.toArray()));

		} catch (SQLException sqlex) {
			sqlex.printStackTrace();
		} finally {
			if (connection != null) {
				try {
					connection.close();
				} catch (SQLException e) {
					e.printStackTrace();
				}
			}
		}
		System.out.println("Connected to DB.");
	}

	private static List<Student> prepareStudentList(ResultSet studentResultSet)
			throws SQLException {
		if (studentResultSet == null) {
			throw new IllegalArgumentException(
					"Null-pointed resultSet argument in prepareStudentList() method.");
		}
		List<Student> studentList = null;
		while (studentResultSet.next()) {
			if (studentList == null) {
				studentList = new LinkedList<Student>();
			}
			Student stud = prepareStudent(studentResultSet);
			studentList.add(stud);
		}
		return studentList;
	}

	private static Student prepareStudent(ResultSet studentResultSet)
			throws SQLException {
		if (studentResultSet == null) {
			throw new IllegalArgumentException(
					"Null-pointed resultSet argument in prepareStudent() method.");
		}

		long id = studentResultSet.getLong("ID");
		String studentName = studentResultSet.getString("NAME");

		String zodiacName = studentResultSet.getString("ZODIAC");
		Zodiac zodiac = Zodiac.valueOf(zodiacName);

		String gradeName = studentResultSet.getString("GRADE");
		Grade grade = new Grade();
		grade.setName(gradeName);

		return new Student(id, studentName, zodiac, grade);
	}
}
